package com.flyer.foster.service.impl;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.intern.InternUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.flyer.exception.BusinessException;
import com.flyer.foster.consts.WechatConst;
import com.flyer.foster.dto.*;
import com.flyer.foster.entity.Area;
import com.flyer.foster.entity.Garden;
import com.flyer.foster.mapper.IGardenMapper;
import com.flyer.foster.service.IAreaService;
import com.flyer.foster.service.IGardenService;
import com.flyer.foster.util.HttpUtils;
import com.flyer.foster.util.WeChatApiUtil;
import com.flyer.foster.util.WxAccessToken;
import okhttp3.Response;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 果园 服务实现类
 * </p>
 *
 * @author flyer
 * @since 2024-05-09
 */
@Service
public class GardenServiceImpl extends ServiceImpl<IGardenMapper, Garden> implements IGardenService {
    @Autowired
    private IAreaService iAreaService;

    @Autowired
    private WeChatApiUtil weChatApiUtil;

    @Autowired
    private QiniuUploadService qiniuUploadService;

    @Override
    public IPage<GardenRespDTO> getPageList(IPage<GardenRespDTO> page, GardenQueryDTO queryDTO) {
        IPage<GardenRespDTO> pageList = baseMapper.getPageList(page, queryDTO);
        for (GardenRespDTO record : pageList.getRecords()) {
            // 获取区域
            List<String> areaNameList = iAreaService.lambdaQuery()
                    .eq(Area::getGardenId, record.getId())
                    .list()
                    .stream().map(Area::getName).collect(Collectors.toList());
            record.setAreaNameList(areaNameList);
        }
        return pageList;
    }

    @Transactional
    @Override
    public boolean addGarden(GardenAddDTO addDTO) {
        int tenantId = StpUtil.getTokenSession().getInt("tenantId");
        Garden garden = new Garden();
        garden.setTenantId(tenantId);
        BeanUtil.copyProperties(addDTO, garden);
        this.save(garden);
        // 生成二维码
        String qrCode = this.generateQRCode(garden.getId());

        return this.lambdaUpdate()
                .set(Garden::getQrCode, qrCode)
                .eq(Garden::getId, garden.getId()).update();
    }

    /**
     * 生成小程序二维码
     *
     * @param gardenId 果园id
     */
    private String generateQRCode(Integer gardenId) {
        WxAccessToken wxAccessToken = WxAccessToken.getInstance(weChatApiUtil, WechatConst.APP_ID, WechatConst.APP_SECRET);
        StringBuilder sb2 = new StringBuilder(200);
        sb2.append("https://api.weixin.qq.com/cgi-bin/wxaapp/createwxaqrcode?access_token=");
        sb2.append(wxAccessToken.getAccess_token());
        Map<String, String> params = new HashMap<>();
        params.put("path", "/pages/subPages/login/index?gardenId=" + gardenId);
        params.put("width", "180");
        System.out.println(params.get("path"));
        Response response = HttpUtils.getResponse(sb2.toString(), params);
        if (response.isSuccessful()) {
            String key = "birdseye-look-mini/temp/" + System.currentTimeMillis() + "_" + gardenId + ".jpg";
            try {
                qiniuUploadService.upload(qiniuUploadService.uploadToken(), key, response.body().bytes());
                return key;
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return "";
    }

    @Transactional
    @Override
    public boolean deleteByIdList(List<Integer> idList) {
        return baseMapper.deleteByIdList(idList);
    }

    @Transactional
    @Override
    public boolean updateGarden(GardenUpdateDTO updateDTO) {
        Garden garden = baseMapper.selectById(updateDTO.getId());
        if (garden == null) {
            throw new BusinessException("无效的果园id");
        }
        return this.lambdaUpdate()
                .set(StrUtil.isNotBlank(updateDTO.getName()), Garden::getName, updateDTO.getName())
                .set(updateDTO.getWorldMapId() != null, Garden::getWorldMapId, updateDTO.getWorldMapId())
                .set(StrUtil.isNotBlank(updateDTO.getWkt()), Garden::getWkt, updateDTO.getWkt())
                .set(StrUtil.isNotBlank(updateDTO.getIcon()), Garden::getIcon, updateDTO.getIcon())
                .set(StrUtil.isNotBlank(updateDTO.getDayBg()), Garden::getDayBg, updateDTO.getDayBg())
                .set(StrUtil.isNotBlank(updateDTO.getNightBg()), Garden::getNightBg, updateDTO.getNightBg())
                .set(updateDTO.getBirdseyeGardenId() != null, Garden::getBirdseyeGardenId, updateDTO.getBirdseyeGardenId())
                .set(updateDTO.getUserId() != null, Garden::getUserId, updateDTO.getUserId())
                .set(StrUtil.isNotBlank(updateDTO.getBaseMapUrl()), Garden::getBaseMapUrl, updateDTO.getBaseMapUrl())
                .set(StrUtil.isNotBlank(updateDTO.getCoverUrl1()), Garden::getCoverUrl1, updateDTO.getCoverUrl1())
                .set(StrUtil.isNotBlank(updateDTO.getCoverUrl1Content()), Garden::getCoverUrl1Content, updateDTO.getCoverUrl1Content())
                .set(StrUtil.isNotBlank(updateDTO.getCoverUrl2()), Garden::getCoverUrl2, updateDTO.getCoverUrl2())
                .set(StrUtil.isNotBlank(updateDTO.getCoverUrl2Content()), Garden::getCoverUrl2Content, updateDTO.getCoverUrl2Content())
                .set(StrUtil.isNotBlank(updateDTO.getCoverUrl3()), Garden::getCoverUrl3, updateDTO.getCoverUrl3())
                .set(StrUtil.isNotBlank(updateDTO.getCoverUrl3Content()), Garden::getCoverUrl3Content, updateDTO.getCoverUrl3Content())
                .set(StrUtil.isNotBlank(updateDTO.getCoverUrl4()), Garden::getCoverUrl4, updateDTO.getCoverUrl4())
                .set(StrUtil.isNotBlank(updateDTO.getCoverUrl4Content()), Garden::getCoverUrl4Content, updateDTO.getCoverUrl4Content())
                .eq(Garden::getId, updateDTO.getId())
                .update();
    }
}
